@@ -137,6 +137,7 @@ void ureg_free_tokens( const struct tgsi_token *tokens );
 void 
 ureg_destroy( struct ureg_program * );
 
+void ureg_set_precise( struct ureg_program *ureg, bool precise );
 
 /***********************************************************************
  * Convenience routine:
@@ -172,11 +173,10 @@ ureg_property(struct ureg_program *ureg, unsigned name, unsigned value);
  */
 
 struct ureg_src
-ureg_DECL_fs_input_cyl_centroid_layout(struct ureg_program *,
+ureg_DECL_fs_input_centroid_layout(struct ureg_program *,
                        enum tgsi_semantic semantic_name,
                        unsigned semantic_index,
                        enum tgsi_interpolate_mode interp_mode,
-                       unsigned cylindrical_wrap,
                        enum tgsi_interpolate_loc interp_location,
                        unsigned index,
                        unsigned usage_mask,
@@ -184,41 +184,25 @@ ureg_DECL_fs_input_cyl_centroid_layout(struct ureg_program *,
                        unsigned array_size);
 
 struct ureg_src
-ureg_DECL_fs_input_cyl_centroid(struct ureg_program *,
+ureg_DECL_fs_input_centroid(struct ureg_program *,
                        enum tgsi_semantic semantic_name,
                        unsigned semantic_index,
                        enum tgsi_interpolate_mode interp_mode,
-                       unsigned cylindrical_wrap,
                        enum tgsi_interpolate_loc interp_location,
                        unsigned array_id,
                        unsigned array_size);
 
-static inline struct ureg_src
-ureg_DECL_fs_input_cyl(struct ureg_program *ureg,
-                       enum tgsi_semantic semantic_name,
-                       unsigned semantic_index,
-                       enum tgsi_interpolate_mode interp_mode,
-                       unsigned cylindrical_wrap)
-{
-   return ureg_DECL_fs_input_cyl_centroid(ureg,
-                                 semantic_name,
-                                 semantic_index,
-                                 interp_mode,
-                                 cylindrical_wrap,
-                                 TGSI_INTERPOLATE_LOC_CENTER, 0, 1);
-}
-
 static inline struct ureg_src
 ureg_DECL_fs_input(struct ureg_program *ureg,
                    enum tgsi_semantic semantic_name,
                    unsigned semantic_index,
                    enum tgsi_interpolate_mode interp_mode)
 {
-   return ureg_DECL_fs_input_cyl_centroid(ureg,
+   return ureg_DECL_fs_input_centroid(ureg,
                                  semantic_name,
                                  semantic_index,
                                  interp_mode,
-                                 0, TGSI_INTERPOLATE_LOC_CENTER, 0, 1);
+                                 TGSI_INTERPOLATE_LOC_CENTER, 0, 1);
 }
 
 struct ureg_src
