@@ -41,6 +41,7 @@
 #include "broadcom/common/v3d_limits.h"
 
 #include "broadcom/simulator/v3d_simulator.h"
+#include "broadcom/compiler/v3d_compiler.h"
 
 struct v3d_job;
 struct v3d_bo;
@@ -165,6 +166,12 @@ struct v3d_sampler_view {
          * raster texture.
          */
         struct pipe_resource *texture;
+
+        /* A serial ID used to identify cases where a new BO has been created
+         * and we need to rebind a sampler view that was created against the
+         * previous BO to to point to the new one.
+         */
+        uint32_t serial_id;
 };
 
 struct v3d_sampler_state {
@@ -280,7 +287,7 @@ struct v3d_ssbo_stateobj {
 
 /* Hash table key for v3d->jobs */
 struct v3d_job_key {
-        struct pipe_surface *cbufs[4];
+        struct pipe_surface *cbufs[V3D_MAX_DRAW_BUFFERS];
         struct pipe_surface *zsbuf;
         struct pipe_surface *bbuf;
 };
@@ -304,6 +311,19 @@ struct v3d_shaderimg_stateobj {
         uint32_t enabled_mask;
 };
 
+struct v3d_perfmon_state {
+        /* The kernel perfmon id */
+        uint32_t kperfmon_id;
+        /* True if at least one job was submitted with this perfmon. */
+        bool job_submitted;
+        /* Fence to be signaled when the last job submitted with this perfmon
+         * is executed by the GPU.
+         */
+        struct v3d_fence *last_job_fence;
+        uint8_t counters[DRM_V3D_MAX_PERF_COUNTERS];
+        uint64_t values[DRM_V3D_MAX_PERF_COUNTERS];
+};
+
 /**
  * A complete bin/render job.
  *
@@ -345,7 +365,7 @@ struct v3d_job {
          * the destination surface.
          */
         uint32_t nr_cbufs;
-        struct pipe_surface *cbufs[4];
+        struct pipe_surface *cbufs[V3D_MAX_DRAW_BUFFERS];
         struct pipe_surface *zsbuf;
         struct pipe_surface *bbuf;
         /** @} */
@@ -413,10 +433,13 @@ struct v3d_job {
          * (either clears or draws) and should be stored.
          */
         uint32_t store;
-        uint32_t clear_color[4][4];
+        uint32_t clear_color[V3D_MAX_DRAW_BUFFERS][4];
         float clear_z;
         uint8_t clear_s;
 
+        /* If TLB double-buffering is enabled for this job */
+        bool double_buffer;
+
         /**
          * Set if some drawing (triangles, blits, or just a glClear()) has
          * been done to the FBO, meaning that we need to
@@ -434,6 +457,8 @@ struct v3d_job {
          */
         bool tf_enabled;
 
+        bool needs_primitives_generated;
+
         /**
          * Current EZ state for drawing. Updated at the start of draw after
          * we've decided on the shader being rendered.
@@ -487,8 +512,6 @@ struct v3d_context {
         /** bitfield of V3D_DIRTY_* */
         uint64_t dirty;
 
-        struct primconvert_context *primconvert;
-
         uint32_t next_uncompiled_program_id;
         uint64_t next_compiled_program_id;
 
@@ -563,6 +586,8 @@ struct v3d_context {
         uint32_t tf_prims_generated;
         uint32_t prims_generated;
 
+        uint32_t n_primitives_generated_queries_in_flight;
+
         struct pipe_poly_stipple stipple;
         struct pipe_clip_state clip;
         struct pipe_viewport_state viewport;
@@ -575,7 +600,9 @@ struct v3d_context {
         struct v3d_bo *current_oq;
         struct pipe_resource *prim_counts;
         uint32_t prim_counts_offset;
-        struct pipe_debug_callback debug;
+        struct util_debug_callback debug;
+        struct v3d_perfmon_state *active_perfmon;
+        struct v3d_perfmon_state *last_perfmon;
         /** @} */
 };
 
@@ -608,7 +635,7 @@ struct v3d_blend_state {
         if (unlikely(V3D_DEBUG & V3D_DEBUG_PERF))       \
                 fprintf(stderr, __VA_ARGS__);           \
         if (unlikely(v3d->debug.debug_message))         \
-                pipe_debug_message(&v3d->debug, PERF_INFO, __VA_ARGS__);    \
+                util_debug_message(&v3d->debug, PERF_INFO, __VA_ARGS__);    \
 } while (0)
 
 static inline struct v3d_context *
@@ -641,6 +668,12 @@ v3d_stream_output_target_get_vertex_count(struct pipe_stream_output_target *ptar
     return v3d_stream_output_target(ptarget)->recorded_vertex_count;
 }
 
+int v3d_get_driver_query_group_info(struct pipe_screen *pscreen,
+                                    unsigned index,
+                                    struct pipe_driver_query_group_info *info);
+int v3d_get_driver_query_info(struct pipe_screen *pscreen, unsigned index,
+                              struct pipe_driver_query_info *info);
+
 struct pipe_context *v3d_context_create(struct pipe_screen *pscreen,
                                         void *priv, unsigned flags);
 void v3d_program_init(struct pipe_context *pctx);
@@ -730,14 +763,23 @@ bool v3d_generate_mipmap(struct pipe_context *pctx,
                          unsigned int first_layer,
                          unsigned int last_layer);
 
+void
+v3d_fence_unreference(struct v3d_fence **fence);
+
 struct v3d_fence *v3d_fence_create(struct v3d_context *v3d);
 
+bool v3d_fence_wait(struct v3d_screen *screen,
+                    struct v3d_fence *fence,
+                    uint64_t timeout_ns);
+
 void v3d_update_primitive_counters(struct v3d_context *v3d);
 
 bool v3d_line_smoothing_enabled(struct v3d_context *v3d);
 
 float v3d_get_real_line_width(struct v3d_context *v3d);
 
+void v3d_ensure_prim_counts_allocated(struct v3d_context *ctx);
+
 void v3d_flag_dirty_sampler_state(struct v3d_context *v3d,
                                   enum pipe_shader_type shader);
 
@@ -745,6 +787,7 @@ void v3d_create_texture_shader_state_bo(struct v3d_context *v3d,
                                         struct v3d_sampler_view *so);
 
 void v3d_get_tile_buffer_size(bool is_msaa,
+                              bool double_buffer,
                               uint32_t nr_cbufs,
                               struct pipe_surface **cbufs,
                               struct pipe_surface *bbuf,
@@ -752,6 +795,17 @@ void v3d_get_tile_buffer_size(bool is_msaa,
                               uint32_t *tile_height,
                               uint32_t *max_bpp);
 
+#ifdef ENABLE_SHADER_CACHE
+struct v3d_compiled_shader *v3d_disk_cache_retrieve(struct v3d_context *v3d,
+                                                    const struct v3d_key *key);
+
+void v3d_disk_cache_store(struct v3d_context *v3d,
+                          const struct v3d_key *key,
+                          const struct v3d_compiled_shader *shader,
+                          uint64_t *qpu_insts,
+                          uint32_t qpu_size);
+#endif /* ENABLE_SHADER_CACHE */
+
 #ifdef v3dX
 #  include "v3dx_context.h"
 #else
