@@ -22,6 +22,7 @@
  */
 
 #include "zink_context.h"
+#include "zink_kopper.h"
 #include "zink_query.h"
 #include "zink_resource.h"
 #include "zink_screen.h"
@@ -75,10 +76,10 @@ clear_in_rp(struct pipe_context *pctx,
 
    if (buffers & PIPE_CLEAR_COLOR) {
       VkClearColorValue color;
-      color.float32[0] = pcolor->f[0];
-      color.float32[1] = pcolor->f[1];
-      color.float32[2] = pcolor->f[2];
-      color.float32[3] = pcolor->f[3];
+      color.uint32[0] = pcolor->ui[0];
+      color.uint32[1] = pcolor->ui[1];
+      color.uint32[2] = pcolor->ui[2];
+      color.uint32[3] = pcolor->ui[3];
 
       for (unsigned i = 0; i < fb->nr_cbufs; i++) {
          if (!(buffers & (PIPE_CLEAR_COLOR0 << i)) || !fb->cbufs[i])
@@ -119,6 +120,19 @@ clear_in_rp(struct pipe_context *pctx,
    struct zink_batch *batch = &ctx->batch;
    zink_batch_rp(ctx);
    VKCTX(CmdClearAttachments)(batch->state->cmdbuf, num_attachments, attachments, 1, &cr);
+   /*
+       Rendering within a subpass containing a feedback loop creates a data race, except in the following
+       cases:
+       • If a memory dependency is inserted between when the attachment is written and when it is
+       subsequently read by later fragments. Pipeline barriers expressing a subpass self-dependency
+       are the only way to achieve this, and one must be inserted every time a fragment will read
+       values at a particular sample (x, y, layer, sample) coordinate, if those values have been written
+       since the most recent pipeline barrier
+
+       VK 1.3.211, Chapter 8: Render Pass
+    */
+   if (ctx->fbfetch_outputs)
+      ctx->base.texture_barrier(&ctx->base, PIPE_TEXTURE_BARRIER_FRAMEBUFFER);
 }
 
 static void
@@ -134,11 +148,15 @@ clear_color_no_rp(struct zink_context *ctx, struct zink_resource *res, const uni
    range.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
 
    VkClearColorValue color;
-   color.float32[0] = pcolor->f[0];
-   color.float32[1] = pcolor->f[1];
-   color.float32[2] = pcolor->f[2];
-   color.float32[3] = pcolor->f[3];
+   color.uint32[0] = pcolor->ui[0];
+   color.uint32[1] = pcolor->ui[1];
+   color.uint32[2] = pcolor->ui[2];
+   color.uint32[3] = pcolor->ui[3];
 
+   if (zink_is_swapchain(res)) {
+      if (!zink_kopper_acquire(ctx, res, UINT64_MAX))
+         return;
+   }
    if (zink_resource_image_needs_barrier(res, VK_IMAGE_LAYOUT_GENERAL, 0, 0) &&
        zink_resource_image_needs_barrier(res, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, 0, 0))
       zink_resource_image_barrier(ctx, res, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, 0, 0);
@@ -458,6 +476,7 @@ zink_clear_buffer(struct pipe_context *pctx,
       zink_batch_no_rp(ctx);
       zink_batch_reference_resource_rw(batch, res, true);
       util_range_add(&res->base.b, &res->valid_buffer_range, offset, offset + size);
+      zink_resource_buffer_barrier(ctx, res, VK_ACCESS_TRANSFER_WRITE_BIT, VK_PIPELINE_STAGE_TRANSFER_BIT);
       VKCTX(CmdFillBuffer)(batch->state->cmdbuf, res->obj->buffer, offset, size, *(uint32_t*)clear_value);
       return;
    }
@@ -533,8 +552,7 @@ zink_fb_clear_util_unpack_clear_color(struct zink_framebuffer_clear_data *clear,
       }
       color->f[3] = clear->color.color.f[3];
    } else {
-      for (unsigned i = 0; i < 4; i++)
-         color->f[i] = clear->color.color.f[i];
+      *color = clear->color.color;
    }
 }
 
@@ -555,8 +573,13 @@ fb_clears_apply_internal(struct zink_context *ctx, struct pipe_resource *pres, i
       else {
          struct pipe_surface *psurf = ctx->fb_state.cbufs[i];
          struct zink_framebuffer_clear_data *clear = zink_fb_clear_element(fb_clear, 0);
-         union pipe_color_union color;
-         zink_fb_clear_util_unpack_clear_color(clear, psurf->format, &color);
+         union pipe_color_union color = clear->color.color;
+
+         if (psurf->format != psurf->texture->format) {
+            uint32_t data[4];
+            util_format_pack_rgba(psurf->format, data, color.ui, 1);
+            util_format_unpack_rgba(pres->format, color.ui, data, 1);
+         }
 
          clear_color_no_rp(ctx, res, &color,
                                 psurf->u.tex.level, psurf->u.tex.first_layer,
