@@ -1080,6 +1080,8 @@ _mesa_initialize_context(struct gl_context *ctx,
       }
       break;
    }
+   ctx->VertexProgram.PointSizeEnabled = ctx->API == API_OPENGLES2;
+   ctx->PointSizeIsOne = GL_TRUE;
 
    ctx->FirstTimeCurrent = GL_TRUE;
 
